var request = require('request');
var async = require('async');
var writeCsv = require('./writeCsv');
var writeSingleEventTypeCsv = require('./writeSingleEventTypeCsv');
var writeBiggestStreams = require('./writeBiggestStreams')

var args, ignoredEvents, streamUrl

var uniqueEvents = {}
var evtOccurrences = {}
var streams = {}
var countAll = 0

var getEvents = function(url, cb) {
  request({
    headers: {
      "Accept": "application/json"
    },
    method: 'GET',
    url: url + "?embed=body",
    auth: {
      user: args.username,
      password: args.password
    }
  }, (err, response, body) => {
    if(err)
      return cb(err, null);

    if(response.statusCode !== 200)
      return cb(`StatusCode:${response.statusCode} on GET ${url}`, null)

    cb(null, body);
  });
}

function getUrlOfStartPage(streamUrl, cb) {
  getEvents(streamUrl, (err, responseString) => {
    if(err)
      throw("Failed to GET " + streamUrl);

    var response = JSON.parse(responseString);
    cb(null, getLink(response, "last").uri)
  })
}

function getLink(response, linkRelation) {
  return response.links.filter((link) => {
    return link.relation === linkRelation
  })[0];
}

function getPageOfEvents(url, handlers, onCaughtUp) {
  getEvents(url, function(err, responseString) {
    if(err)
      throw err
    
    try {
      var response = JSON.parse(responseString);
      var events = response.entries || [];
      var sortedEvents = events.reverse();

      async.eachSeries(sortedEvents, (evt, cb) => {
        handlers(evt, cb)
      }, () => {
        var linkToNextPage = getLink(response, "previous")
        if(linkToNextPage) {
          getPageOfEvents(linkToNextPage.uri, handlers, onCaughtUp, getEvents);
        }else{
          onCaughtUp();
        }
      })
    } catch(err) {
      console.error("ERROR")      
      console.error(err)
    }
  });
}

var handler = (evt, cb) => {
  countAll++

  if(!uniqueEvents[evt.eventType])
    uniqueEvents[evt.eventType] = {count: 0, size: 0}

  uniqueEvents[evt.eventType].count++
  if(evt.data)
    uniqueEvents[evt.eventType].size+=evt.data.length

  cb()
}

var biggestStreamsHandler = (evt, cb) => {
  if(!streams[evt.streamId])
    streams[evt.streamId] = {count: 0}

  streams[evt.streamId].count++

  cb()
}

var singleEventTypeHandler = (evt, cb) => {
  if(evt.eventType === args.type) {
    if(!evtOccurrences[evt.streamId])
      evtOccurrences[evt.streamId] = {count: 0}

    evtOccurrences[evt.streamId].count++
  }

  cb()
}

var onCaughtUp = () => {
  if(args.countperstream){
    writeBiggestStreams(streams)
  } else if(args.type.length === 0) {
    writeCsv(uniqueEvents)
  }  else {
    writeSingleEventTypeCsv(evtOccurrences)
  }
  console.error("Finished")
}

module.exports = function(_args) {
  args = _args
  if(args.readstream)
    streamUrl = `http://${args.ipaddress}:${args.port}/streams/${args.readstream}`;
  else
    streamUrl = `http://${args.ipaddress}:${args.port}/streams/%24all`;
  
  var h
  
  if(args.countperstream){
    console.error("Getting number of events per stream")
    h = biggestStreamsHandler
  } else if(args.type.length === 0) {
    console.error("Reading All Events...")
    h = handler
  }else{
    console.error("Reading", args.type, "Events...")
    h = singleEventTypeHandler
  }
  getUrlOfStartPage(streamUrl, (err, url) => {
    if(err)
      throw err

    getPageOfEvents(url, h, onCaughtUp, getEvents)
  })
}
